package net.sf.javaprinciples.data.transformer;

import java.util.List;

import org.apache.commons.lang.StringUtils;

import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.jcc.model.parser.uml2.ElementStore;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Factory to create a {@link ModelElementMapper} from the stereotype of a given
 * model element.
 *
 * @author Kay Chevalier
 */
public class ModelElementMapperFactory<T,U> extends ModelElementFactory implements MapperFactory
{
    private ModelElementMapperBuilderBeanFactory modelElementMapperBuilderBeanFactory;


    @Override
    public Mapper<T,U> createMapper(String mapperGuid, ElementStore store, TransformerFactory.DIRECTION_TYPE direction)
    {
        return (Mapper<T,U>)createModelElementMapper(mapperGuid, store, direction);
    }

    @Override
    protected ModelElementMapper<T,U> createModelElementMapper(String mapperGuid, ElementStore store, TransformerFactory.DIRECTION_TYPE direction)
    {
        ModelElement modelElement = store.get(mapperGuid);
        String stereotype = getModelElementStereotype(modelElement);
        String implementationTag = ModelElementMapperHelper.retrieveModelElementTagValue(modelElement, IMPLEMENTATION_TAG);

        if (StringUtils.isEmpty(implementationTag))
        {
           implementationTag = stereotype;
        }
        if (implementationTag == null)
        {
            throw new UnexpectedException("No implementation for " + ModelElementMapperHelper.getPropertyNameFromElement(modelElement));
        }

        ModelElementMapper<T,U> modelElementMapper = getModelElementMapper(implementationTag);
        modelElementMapper.setStore(store);
        modelElementMapper.setModelElement(modelElement);
        List<ModelElement> sourceModelElements = retrieveSourceFromAssociations(modelElement, direction, store);
        List<ModelElement> destinationModelElements = retrieveDestinationFromAssociations(modelElement, direction, store);
        modelElementMapper.setSourceModelElements(sourceModelElements);
        modelElementMapper.setDestinationModelElements(destinationModelElements);

        ModelElementMapperBuilder builder = modelElementMapperBuilderBeanFactory.createModelElementMapperBuilder(implementationTag + ".builder");
        builder.build(modelElementMapper, direction);

        return modelElementMapper;
    }

    public void setModelElementMapperBuilderBeanFactory(ModelElementMapperBuilderBeanFactory modelElementMapperBuilderBeanFactory)
    {
        this.modelElementMapperBuilderBeanFactory = modelElementMapperBuilderBeanFactory;
    }

}
