package net.sf.javaprinciples.data.transformer;

import java.util.ArrayList;
import java.util.List;

import au.com.sparxsystems.Association;
import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.jcc.model.parser.uml2.ElementStore;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Initialise the name and value source model elements
 *
 * @author Warwick Slade
 */
public class ValuePairAttributeMapperBuilder<T,U> extends AttributeMapperBuilder<ValuePairAttributeMapper<T,U>, T, U>
{
    private static final String ATTRIBUTE_MAPPER = "sourceAttributeMapper";

    private SourceDestinationAttributeMapperBeanFactory sourceDestinationAttributeMapperBeanFactory;

    @Override
    public void build(ValuePairAttributeMapper<T,U> mapperModelElement, TransformerFactory.DIRECTION_TYPE direction)
    {
        super.build(mapperModelElement, direction);

        List<Association> associations = mapperModelElement.getModelElement().getAssociations();

        List<ModelElement> elements = new ArrayList<ModelElement>();

        if (associations == null)
        {
            return;
        }

        boolean nameSet = false;
        boolean valueSet = false;
        for (Association association : associations)
        {
            String stereotype = findAssociationStereotype(association);

            if ("name".equals(stereotype))
            {
                ModelElementSourceDestinationMapper sourceMapper =
                        makeSourceDestinationMapper(association, mapperModelElement.store, ATTRIBUTE_MAPPER);

                mapperModelElement.setNameModelElementMapper(sourceMapper);
                nameSet = true;
            }
            else if ("value".equals(stereotype))
            {
                ModelElementSourceDestinationMapper sourceMapper =
                        makeSourceDestinationMapper(association, mapperModelElement.store, ATTRIBUTE_MAPPER);

                mapperModelElement.setValueModelElementMapper(sourceMapper);
                valueSet = true;
            }
        }
        if (!valueSet)
        {
            AbstractSourceDestinationAttributeMapper sourceMapper =
                    (AbstractSourceDestinationAttributeMapper)sourceDestinationAttributeMapperBeanFactory.createModelElementMapper("passThroughSourceAttributeMapper");
            mapperModelElement.setValueModelElementMapper(sourceMapper);
        }
        if (!nameSet)
        {
            throw new UnexpectedException("Missing name association on : " + mapperModelElement.getModelElement().getGuid());
        }
    }

    protected ModelElementSourceDestinationMapper makeSourceDestinationMapper(Association association,
                                                                              ElementStore store,
                                                                              String beanName)
    {
        AbstractSourceDestinationAttributeMapper sourceMapper =
                (AbstractSourceDestinationAttributeMapper)sourceDestinationAttributeMapperBeanFactory.createModelElementMapper(beanName);

        sourceMapper.setObjectTypeMapper(objectTypeMapper);
        sourceMapper.setMappingElement(retrieveAttributeFromAssociation(association, store));

        return sourceMapper;
    }

    public void setSourceDestinationAttributeMapperBeanFactory(SourceDestinationAttributeMapperBeanFactory sourceDestinationAttributeMapperBeanFactory)
    {
        this.sourceDestinationAttributeMapperBeanFactory = sourceDestinationAttributeMapperBeanFactory;
    }
}
