package net.sf.javaprinciples.data.transformer.attributemapper;

import java.util.List;

import org.omg.uml.LiteralSpecification;
import org.omg.uml.LiteralString;
import org.omg.uml.Property;
import org.omg.uml.ValueSpecification;

import net.sf.javaprinciples.core.UnexpectedException;
import net.sf.javaprinciples.data.transformer.SourceDestinationAttributeMapper;
import net.sf.jcc.model.parser.uml2.ModelElement;

/**
 * Mapper to handle the assignment of a default value into a property.
 *
 * @author Kay Chevalier
 */
public class DefaultValueMapper extends SourceDestinationAttributeMapper
{
    @Override
    public Object getAttributeFromObject(List<ModelElement> modelElements, Object parentObject)
    {
        Object mappingElement = this.mappingElement.getElement();
        if (mappingElement == null || !(mappingElement instanceof Property))
        {
            throw new UnexpectedException("DefaultValueMapper expected configured mapping element of type org.omg.uml.Property");
        }
        Property property = (Property) mappingElement;
        List<ValueSpecification> defaultValueList = property.getDefaultValue();
        if (defaultValueList.size() != 1)
        {
            throw new UnexpectedException(
                    String.format("DefaultValueMapper expected exactly one default value, but found %s.",
                                  defaultValueList.size()));
        }
        ValueSpecification defaultValue = defaultValueList.get(0);
        if (!(defaultValue instanceof LiteralSpecification))
        {
            throw new UnexpectedException(
                    String.format(
                            "DefaultValueMapper expected default value of type org.omg.uml.LiteralSpecification, but found %s.",
                            defaultValue.getClass().getSimpleName()));
        }
        return ((LiteralString) defaultValue).getValue();
    }
}
